import sys
import tornado
import zmq
from tornado.web import Application
from tornado.ioloop import IOLoop

from sql_injection_fsm import SQLInjectionFSM
from tfw.config import tfwenv
from tfw.config.logs import logging
from tfw.networking.controller_connector import ControllerConnector
from tfw.networking.serialization import deserialize_all, serialize_all

log = logging.getLogger(__name__)
from tfw.networking.server.zmq_websocket_handler import FSMManagingSocketHandler


def zmq_callback(stream, msg_parts):
    key, data = deserialize_all(*msg_parts)
    if key == 'test':
        stream.send_multipart(serialize_all(key, 'OK'))
    if key == 'solution_check':
        stream.send_multipart(serialize_all(key, {
            'solved': True,
            'message': 'solved'
        }))


if __name__ == '__main__':
    fsm = SQLInjectionFSM()
    controller_connector = ControllerConnector()
    controller_connector.register_callback(zmq_callback)
    routes = [
        (r'/ws', FSMManagingSocketHandler, {'fsm': fsm}),
    ]
    application = Application(
        routes,
        autoreload=True
    )

    application.listen(tfwenv.WEB_PORT)
    log.debug('Python version: {}'.format(sys.version[:5]))
    log.debug('Tornado version: {}'.format(tornado.version))
    log.debug('ZeroMQ version: {}'.format(zmq.zmq_version()))
    log.debug('PyZMQ version: {}'.format(zmq.pyzmq_version()))
    log.info('Tornado application listening on port {}'.format(tfwenv.WEB_PORT))
    IOLoop.instance().start()
